@@ -30,7 +30,7 @@ use namespace::clean;
 
 with 'Lintian::Check';
 
-sub files {
+sub visit_installed_files {
     my ($self, $file) = @_;
 
     if ($file->name =~ m,/icons/[^/]+/(\d+)x(\d+)/(?!animations/).*\.png$,){
